//--------------------------------------------------------------------------
// Guitar Practice
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CGPEqualizer.
//
//	Description:  	This unit implements the guitar practice equalizer
//
//
//
// Author(s):		FeMann
// Created:		19.10.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//--------------------------------------------------------------------------
// Constructor and Destructor
//--------------------------------------------------------------------------

// Constructor
//
constructor CGPEqualizer.Create(mySoundSystem: CGPSoundSystem);
begin
	inherited Create;

   // Set backward reference to sound system
   m_soundSystem := mySoundSystem;

	// Reset the equalizer
	Reset;
end;

// Destructor
//
destructor CGPEqualizer.Destroy;
begin
	try
		// Reset backward reference to sound system
       m_soundSystem := nil;
   finally
   	inherited;
   end;
end;

//--------------------------------------------------------------------------
// Public methods
//--------------------------------------------------------------------------

// Reset the equalizer
//
procedure CGPEqualizer.Reset;
begin
	// Reset parameters
	FillChar(m_parameters,SizeOf(m_parameters),0);

   // Setup initial parameters
	m_parameters.fGain := 0;
	m_parameters.fQ := 0;
	m_parameters.fBandwidth := 2.5;
	m_parameters.lChannel := BASS_BFX_CHANALL;
end;

// Init the equalizer
//
procedure CGPEqualizer.Initialize;
var
	i: integer;
begin
	// Initialize the bands
   initBand(0,60);
   initBand(1,170);
   initBand(2,310);
   initBand(3,600);
   initBand(4,1000);
   initBand(5,3000);
   initBand(6,6000);
   initBand(7,12000);
   initBand(8,14000);
   initBand(9,16000);

   // Reset the gain
   for i := 0 to 9 do
   	updateBand(i, m_bands[i]);
end;

//--------------------------------------------------------------------------
// Private methods
//--------------------------------------------------------------------------

// Initialize the band
//
procedure CGPEqualizer.initBand(number: integer; centerFrequency: float);
begin
	// Setup band parameters
   m_parameters.lBand := number;
   m_parameters.fCenter := centerFrequency;

	// Delegate to sound system
   SoundSystem.SetEqualizerParameters(m_parameters);
end;

// Update the given band
//
procedure CGPEqualizer.updateBand(number: integer; gain: integer);
begin
   // Select the band
	m_parameters.lBand := number;

   // Get the parameters from sound system
   if SoundSystem.GetEqualizerParameters(m_parameters) then
	begin
   	// Calculate gain
       m_parameters.fGain := 10 - gain;

       // Set the parameters
       SoundSystem.SetEqualizerParameters(m_parameters);
	end;
end;

//--------------------------------------------------------------------------
// Property methods
//--------------------------------------------------------------------------

// Get the gain of band 1
//
function CGPEqualizer.getBand1Gain: integer;
begin
	result := m_bands[0];
end;

// Set the gain of band 1
//
procedure CGPEqualizer.setBand1Gain(val: integer);
begin
	// Store the band
   m_bands[0] := val;

   // Update the band
   updateBand(0,m_bands[0]);
end;

// Get the gain of band 2
//
function CGPEqualizer.getBand2Gain: integer;
begin
	result := m_bands[1];
end;

// Set the gain of band 1
//
procedure CGPEqualizer.setBand2Gain(val: integer);
begin
	// Store the band
   m_bands[1] := val;

   // Update the band
   updateBand(1,m_bands[1]);
end;

// Get the gain of band 3
//
function CGPEqualizer.getBand3Gain: integer;
begin
	result := m_bands[2];
end;

// Set the gain of band 3
//
procedure CGPEqualizer.setBand3Gain(val: integer);
begin
	// Store the band
   m_bands[2] := val;

   // Update the band
   updateBand(2,m_bands[2]);
end;

// Get the gain of band 4
//
function CGPEqualizer.getBand4Gain: integer;
begin
	result := m_bands[3];
end;

// Set the gain of band 4
//
procedure CGPEqualizer.setBand4Gain(val: integer);
begin
	// Store the band
   m_bands[3] := val;

   // Update the band
   updateBand(3,m_bands[3]);
end;

// Get the gain of band 5
//
function CGPEqualizer.getBand5Gain: integer;
begin
	result := m_bands[4];
end;

// Set the gain of band 5
//
procedure CGPEqualizer.setBand5Gain(val: integer);
begin
	// Store the band
   m_bands[4] := val;

   // Update the band
   updateBand(4,m_bands[4]);
end;

// Get the gain of band 6
//
function CGPEqualizer.getBand6Gain: integer;
begin
	result := m_bands[5];
end;

// Set the gain of band 6
//
procedure CGPEqualizer.setBand6Gain(val: integer);
begin
	// Store the band
   m_bands[5] := val;

   // Update the band
   updateBand(5,m_bands[5]);
end;

// Get the gain of band 7
//
function CGPEqualizer.getBand7Gain: integer;
begin
	result := m_bands[6];
end;

// Set the gain of band 7
//
procedure CGPEqualizer.setBand7Gain(val: integer);
begin
	// Store the band
   m_bands[6] := val;

   // Update the band
   updateBand(6,m_bands[6]);
end;

// Get the gain of band 8
//
function CGPEqualizer.getBand8Gain: integer;
begin
	result := m_bands[7];
end;

// Set the gain of band 8
//
procedure CGPEqualizer.setBand8Gain(val: integer);
begin
	// Store the band
   m_bands[7] := val;

   // Update the band
   updateBand(7,m_bands[7]);
end;

// Get the gain of band 9
//
function CGPEqualizer.getBand9Gain: integer;
begin
	result := m_bands[8];
end;

// Set the gain of band 9
//
procedure CGPEqualizer.setBand9Gain(val: integer);
begin
	// Store the band
   m_bands[8] := val;

   // Update the band
   updateBand(8,m_bands[8]);
end;

// Get the gain of band 10
//
function CGPEqualizer.getBand10Gain: integer;
begin
	result := m_bands[9];
end;

// Set the gain of band 10
//
procedure CGPEqualizer.setBand10Gain(val: integer);
begin
	// Store the band
   m_bands[9] := val;

   // Update the band
   updateBand(9,m_bands[9]);
end;

// Get the equalizer settings as string
//
function CGPEqualizer.getAsString: string;
var
	list: CDelimitedStringList;
   i: integer;
begin
	// Create the delimited string list
	list := CDelimitedStringList.Create;
	try
		// Add the values to the list
		for i := 0 to 9 do
			list.Add(IntToStr(m_bands[i]));

		// Return the delimiter text
		result := list.DelimitedText;
	finally
		// Free the list
		FreeAndNil(list);
	end;
end;

// Set the equalizer settings as string
//
procedure CGPEqualizer.setAsString(val: string);
var
	list: CDelimitedStringList;
   i: integer;
begin
	// Reset the files
   Reset;

	// Create the delimited string list
	list := CDelimitedStringList.Create;
	try
		// Set the delimited value
		list.DelimitedText := val;

       // Add the bands
		for i := 0 to list.Count - 1 do
			m_bands[i] := StrToInt(list[i]);

       // Initialize
       Initialize;
	finally
		// Free the list
		FreeAndNil(list);
	end;
end;

