//--------------------------------------------------------------------------
// Guitar Practice
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CGPFile.
//
//	Description:  	This unit implements the guitar practice file
//
//
//
// Author(s):		FeMann
// Created:		30.08.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CGPFile.Create(myManager: CGPManager);
begin
	inherited Create;

   // Set manager backward reference
	m_manager := myManager;

	// Reset the file
	Reset;
end;

// Destructor
//
destructor CGPFile.Destroy;
begin
	try
   	// Reset manager reference
       m_manager := nil;
   finally
		inherited;
   end;
end;

//------------------------------------------------------------------------------
// Public declarations
//------------------------------------------------------------------------------

// Reset the file
//
procedure CGPFile.Reset;
begin
	m_name := '';
	m_title := '';
end;

// Load the file
//
procedure CGPFile.Load(fileName: string);
begin
	// Check if file exists
	if not FileExists(fileName) then
		exit;

	// Create the file
	m_file := TIniFile.Create(fileName);
	try
		// Load file data
		loadFileData;
	finally
		// Free file
		FreeAndNil(m_file);

		// Store the filename
		m_name := fileName;
	end;
end;

// Save the file
//
procedure CGPFile.Save(fileName: string);
begin
	// Create the file
	m_file := TIniFile.Create(fileName);
	try
		// Save file data
		saveFileData;

		// Update the file
		m_file.UpdateFile;
	finally
		// Free file
		FreeAndNil(m_file);

		// Store the filename
		m_name := fileName;
	end;
end;

//------------------------------------------------------------------------------
// Private declarations
//------------------------------------------------------------------------------

// Save the data
//
procedure CGPFile.saveFileData;
var
	i: integer;
	segment: CGPSoundSegment;
begin
	// Write title
	m_file.WriteString(GP_FILE_SECTION_GENERAL, GP_FILE_VALUE_TITLE, Title);

   // Write the equalizer settings
	m_file.WriteString(GP_FILE_SECTION_GENERAL, GP_FILE_VALUE_EQSETTINGS, m_manager.SoundSystem.Equalizer.AsString);

	// Write the sound segments
	for i := 0 to m_manager.SoundSegments.Count - 1 do
	begin
		// Get the sound segment
		segment := m_manager.SoundSegments[i];

		// Write the sound segment to file
		m_file.WriteString(GP_FILE_SECTION_SOUNDSEGMENTS, GP_FILE_VALUE_SOUNDSEGMENT + IntToStr(i), segment.AsString);
	end;
end;

// Load the data
//
procedure CGPFile.loadFileData;
var
	i: integer;
	values: TStringList;
	segment: CGPSoundSegment;
	sectionValue: string;
	value: string;
begin
	// Clear sound segments
	m_manager.SoundSegments.Reset;

	// Read title
	Title := m_file.ReadString(GP_FILE_SECTION_GENERAL, GP_FILE_VALUE_TITLE, GP_FILE_DEFAULT_TITLE);

   // Read the equalizer settings
   m_manager.SoundSystem.Equalizer.AsString := m_file.ReadString(GP_FILE_SECTION_GENERAL, GP_FILE_VALUE_EQSETTINGS, GP_FILE_DEFAULT_EQSETTINGS);

	// Create value list
	values := TStringList.Create;
	try
		// Read the section values
		m_file.ReadSectionValues(GP_FILE_SECTION_SOUNDSEGMENTS,values);

		// Add the items so sound segment list
		for i := 0 to values.Count - 1 do
		begin
			// Add new sound segment
			segment := m_manager.SoundSegments.AddSoundSegment;

			// Get the section value
			sectionValue := values[i];

			// Get the value
			value := Copy(sectionValue,Pos('=',sectionValue) + 1,Length(sectionValue) - Pos('=',sectionValue));

			// Define the sound segment from sting
			segment.AsString := value;
		end;
	finally
		// Free the values list
		FreeAndNil(values);
	end;
end;

//------------------------------------------------------------------------------
// Property declarations
//------------------------------------------------------------------------------

