//--------------------------------------------------------------------------
// Guitar Practice
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CGPOptions.
//
//	Description:  	This unit implements the guitar practice options
//
//
//
// Author(s):		FeMann
// Created:		30.08.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CGPOptions.Create(myManager: CGPManager);
begin
	inherited Create;

   // Set manager backward reference
	m_manager := myManager;

	// Create the encoder defaults
	m_encoderDefaults := TStringList.Create;
	
	// Reset the options
	Reset;
end;

// Destructor
//
destructor CGPOptions.Destroy;
begin
	try
		// Free the encoder defaults
		FreeAndNil(m_encoderDefaults);
		
   	// Reset manager reference
       m_manager := nil;
   finally
		inherited;
   end;
end;

//------------------------------------------------------------------------------
// Public declarations
//------------------------------------------------------------------------------

// Reset the options
//
procedure CGPOptions.Reset;
begin
	m_left := -1;
	m_top := -1;
	m_width := -1;
	m_height := -1;
   m_language := '';
	m_lastOpenFilename := '';
   m_lastOpenFiles := '';
   m_currentMP3Directory := '';
	m_currentGPFDirectory := '';
	m_encoder := '';
	m_encoderCommandLine := '';
	m_encoderOutputDirectory := '';
	m_keepFileHistory := GP_OPTIONFILE_DEFAULT_KEEPFILEHISTORY;
	m_encoderDefaults.Clear;
end;

// Load the file
//
procedure CGPOptions.Load;
begin
	// Create the file
	m_file := TIniFile.Create(OptionsFileName);
	try
		// Load options
		loadOptions;
	finally
		// Free file
		FreeAndNil(m_file);
	end;
end;

// Save the file
//
procedure CGPOptions.Save;
begin
	// Create the file
	m_file := TIniFile.Create(OptionsFileName);
	try
   	// Save general options
		saveOptions;

		// Update the file
		m_file.UpdateFile;
   finally
       // Free file
		FreeAndNil(m_file);
   end;
end;

//------------------------------------------------------------------------------
// Private declarations
//------------------------------------------------------------------------------

// Save the general options
//
procedure CGPOptions.saveOptions;
begin
	m_file.WriteInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_LEFT, Left);
	m_file.WriteInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_TOP, Top);
	m_file.WriteInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_WIDTH, Width);
	m_file.WriteInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_HEIGHT, Height);
   m_file.WriteString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_LASTOPENFILENAME, LastOpenFileName);
	m_file.WriteString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_LANGUAGE, Language);
   m_file.WriteString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_LASTOPENFILES, LastOpenFiles);
	m_file.WriteString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_CURRENTMP3DIR, CurrentMP3Directory);
	m_file.WriteString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_CURRENTGPFDIR, CurrentGPFDirectory);
	m_file.WriteString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_ENCODER, Encoder);
	m_file.WriteString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_ENCODERCOMMANDLINE, EncoderCommandLine);
	m_file.WriteString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_ENCODEROUTPUTDIRECTORY, EncoderOutputDirectory);
	m_file.WriteInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_KEEPFILEHISTORY, KeepFileHistory);

	// Save the encoder defaults
	saveEncoderDefaults;
end;

// Load the general options
//
procedure CGPOptions.loadOptions;
begin
	Left := m_file.ReadInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_LEFT, GP_OPTIONFILE_DEFAULT_LEFT);
	Top := m_file.ReadInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_TOP, GP_OPTIONFILE_DEFAULT_TOP);
	Width := m_file.ReadInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_WIDTH, GP_OPTIONFILE_DEFAULT_WIDTH);
	Height := m_file.ReadInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_HEIGHT, GP_OPTIONFILE_DEFAULT_HEIGHT);
	LastOpenFileName := m_file.ReadString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_LASTOPENFILENAME, GP_OPTIONFILE_DEFAULT_LASTOPENFILENAME);
	Language := m_file.ReadString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_LANGUAGE, GP_OPTIONFILE_DEFAULT_LANGUAGE);
	LastOpenFiles := m_file.ReadString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_LASTOPENFILES, GP_OPTIONFILE_DEFAULT_LASTOPENFILES);
	CurrentMP3Directory := m_file.ReadString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_CURRENTMP3DIR, GP_OPTIONFILE_DEFAULT_CURRENTMP3DIR);
	CurrentGPFDirectory := m_file.ReadString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_CURRENTGPFDIR, GP_OPTIONFILE_DEFAULT_CURRENTGPFDIR);
	Encoder := m_file.ReadString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_ENCODER, GP_OPTIONFILE_DEFAULT_ENCODER);
	EncoderCommandLine := m_file.ReadString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_ENCODERCOMMANDLINE, GP_OPTIONFILE_DEFAULT_ENCODERCOMMANDLINE);
	EncoderOutputDirectory := m_file.ReadString(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_ENCODEROUTPUTDIRECTORY, GP_OPTIONFILE_DEFAULT_ENCODEROUTPUTDIRECTORY);
	KeepFileHistory := m_file.ReadInteger(GP_OPTIONFILE_SECTION_GENERAL, GP_OPTIONFILE_VALUE_KEEPFILEHISTORY, GP_OPTIONFILE_DEFAULT_KEEPFILEHISTORY);

	// Load the encoder defaults
	loadEncoderDefaults;
end;

// Init the encoder defaults
//
procedure CGPOptions.initEncoderDefaults;
begin
	// Lame encoder
	if not HasEncoder[GP_ENCODER_DEFAULT_LAME_EXE_NAME] then
		m_encoderDefaults.Add(GP_ENCODER_DEFAULT_LAME_EXE_NAME + ';' + GP_ENCODER_DEFAULT_LAME_COMMAND_LINE); 

	// Ogg vorbis encoder
	if not HasEncoder[GP_ENCODER_DEFAULT_OGGVORBIS_EXE_NAME] then
		m_encoderDefaults.Add(GP_ENCODER_DEFAULT_OGGVORBIS_EXE_NAME + ';' + GP_ENCODER_DEFAULT_OGGVORBIS_COMMAND_LINE); 
end;

// Save the encoder defaults
//
procedure CGPOptions.saveEncoderDefaults;
var
	i: integer;
begin
	// Write the encoder defaults to file
	for i := 0 to m_encoderDefaults.Count - 1 do
		m_file.WriteString(GP_OPTIONFILE_SECTION_ENCODERDEFAULTS, GP_OPTIONFILE_VALUE_ENCODERDEFAULT + IntToStr(i), m_encoderDefaults[i]);
end;

// Load the encoder defaults
//
procedure CGPOptions.loadEncoderDefaults;
var
	i: integer;
	values: TStringList;
	sectionValue: string;
	value: string;
begin
	// Clear encoder defaults
	m_encoderDefaults.Clear;
	
	// Create value list
	values := TStringList.Create;
	try
		// Read the section values
		m_file.ReadSectionValues(GP_OPTIONFILE_SECTION_ENCODERDEFAULTS,values);

		// Add the items to the encoder defaults list
		for i := 0 to values.Count - 1 do
		begin
			// Get the section value
			sectionValue := values[i];

			// Get the value
			value := Copy(sectionValue,Pos('=',sectionValue) + 1,Length(sectionValue) - Pos('=',sectionValue));

			// Add the encoder defaults
			m_encoderDefaults.Add(value);
		end;
	finally
		// Free the values list
		FreeAndNil(values);

		// Initialize the encoder defaults
		initEncoderDefaults;
	end;
end;

//------------------------------------------------------------------------------
// Property declarations
//------------------------------------------------------------------------------

// Get the options filename
//
function CGPOptions.getOptionsFileName: string;
begin
	result := PathIncludeTrailingBackslash(ExtractFilePath(ParamStr(0))) + GP_OPTIONFILE_FILENAME;
end;

// Determine if export allowed
//
function CGPOptions.getCanExport: boolean;
begin
	result := (EncoderOutputDirectory <> '') and DirectoryExists(EncoderOutputDirectory) and (Encoder <> '') and FileExists(Encoder) and (EncoderCommandLine <> '');
end;

// Get the encoder default command line by name
//
function CGPOptions.getEncoderDefaultCommandLineByName(name: string): string;
var
	ucName: string;
	encoderName: string;
	list: CDelimitedStringList;
	i: integer;
begin
	// Get the uppercase name
	ucName := UpperCase(name);

	// Create the delimited string list
	list := CDelimitedStringList.Create;
	try
		for i := 0 to m_encoderDefaults.Count - 1 do
		begin
			// Set the delimited value
			list.DelimitedText := m_encoderDefaults[i];
			
			// Get the encoder name
			encoderName := UpperCase(list[0]);

			// Compare
			if (encoderName = ucName) then
           begin
				// Return command line
				result := list[1];

				// Nothing more to do
				exit;
			end;
		end;
	finally
		// Free the list
		FreeAndNil(list);
	end;
end;

// Determine if there is an encoder with given name
//
function CGPOptions.getHasEncoder(name: string): boolean;
begin
	result := (getEncoderDefaultCommandLineByName(name) <> '');
end;

// Get the encoder output directory
//
function CGPOptions.getEncoderOutputDirectory: string;
begin
	result := PathIncludeTrailingBackslash(m_encoderOutputDirectory);
end;

// Set the encoder output directory
//
procedure CGPOptions.setEncoderOutputDirectory(val: string);
begin
	m_encoderOutputDirectory := PathIncludeTrailingBackslash(val); 
end;
