//--------------------------------------------------------------------------
// Guitar Practice
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CGPSoundSegment.
//
//	Description:  	This unit implements the guitar practice sound segment
//
//
//
// Author(s):		FeMann
// Created:		30.08.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//--------------------------------------------------------------------------
// Constructor and Destructor
//--------------------------------------------------------------------------

// Constructor
//
constructor CGPSoundSegment.Create(mySegments: CGPSoundSegments);
begin
	inherited Create;

   // Set backward reference to segments
   m_segments := mySegments;

	// Reset the segment
	Reset;
end;

// Destructor
//
destructor CGPSoundSegment.Destroy;
begin
	try
		// Reset backward reference to segment
       m_segments := nil;
   finally
   	inherited;
   end;
end;

//--------------------------------------------------------------------------
// Public methods
//--------------------------------------------------------------------------

// Reset the segment
//
procedure CGPSoundSegment.Reset;
begin
	m_filename := '';
	m_description := '';
	m_loop := false;
	m_loopStart := -1;
	m_loopEnd := -1;
	m_seconds := -1;
	m_sampleRate := -1;
	m_volume := -1;
	m_tempo := -1;
	m_pitch := -1;
	m_panning := -1;
end;

//--------------------------------------------------------------------------
// Private methods
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
// Property methods
//--------------------------------------------------------------------------

// Get the play time
//
function CGPSoundSegment.getPlaytime: currency;
begin
	if CanLoop then
   	result := (LoopEnd - LoopStart)
   else
   	result := Seconds;
end;

// Check if segment can loop
//
function CGPSoundSegment.getCanLoop: boolean;
begin
	result := (Loop) and (LoopStart <> LoopEnd) and (LoopStart < LoopEnd);
end;

// Get the short file name
//
function CGPSoundSegment.getShortFileName: string;
begin
	result := ExtractFileName(FileName);
end;

// Get the current segment as string
//
function CGPSoundSegment.getAsString: string;
var
	list: TStringList;
begin
	// Create the string list
	list := TStringList.Create;
	try
		// Add the values to the list
		list.Add(FileName);
		list.Add(Description);
		list.Add(BooleanToString(Loop));
		list.Add(CurrToStr(LoopStart));
		list.Add(CurrToStr(LoopEnd));
		list.Add(IntToStr(SampleRate));
		list.Add(IntToStr(Volume));
		list.Add(IntToStr(Tempo));
		list.Add(IntToStr(Pitch));
		list.Add(IntToStr(Panning));

		// Return the delimiter text
		result := list.CommaText;
	finally
		// Free the list
		FreeAndNil(list);
	end;
end;

// Set the current segment as string
//
procedure CGPSoundSegment.setAsString(val: string);
var
	list: TStringList;
begin
	// Create the string list
	list := TStringList.Create;
	try
		// Set the delimited value
		list.CommaText := val;

		// Validate (list has to be exact 10 entries
		if (list.Count <> 10) then
			exit;

		// Load the properties from the list
		FileName := list[0];
		Description := list[1];
		Loop := StringToBoolean(list[2]);
		LoopStart := StrToCurr(list[3]);
		LoopEnd := StrToCurr(list[4]);
		SampleRate := StrToInt(list[5]);
		Volume := StrToInt(list[6]);
		Tempo := StrToInt(list[7]);
		Pitch := StrToInt(list[8]);
		Panning := StrToInt(list[9]);
	finally
		// Free the list
		FreeAndNil(list);
	end;
end;
