//--------------------------------------------------------------------------
// Guitar Practice
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CGPSoundSegments.
//
//	Description:  	This unit implements the guitar practice sound segments
//
//
//
// Author(s):		FeMann
// Created:		30.08.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//--------------------------------------------------------------------------
// Constructor and Destructor
//--------------------------------------------------------------------------

// Constructor
//
constructor CGPSoundSegments.Create(myManager: CGPManager);
begin
	inherited Create;

   // Set backward reference to manager
   m_manager := myManager;

   // Create segment list
	m_segments := TObjectList.Create(true);
end;

// Destructor
//
destructor CGPSoundSegments.Destroy;
begin
	try
   	// Reset
       Reset;

       // Free the segments
       FreeAndNil(m_segments);

       // Reset backward reference to manager
       m_manager := nil;
   finally
   	inherited;
   end;
end;

//--------------------------------------------------------------------------
// Public methods
//--------------------------------------------------------------------------

// Reset the segments
//
procedure CGPSoundSegments.Reset;
begin
	// Clear the segments
	m_segments.Clear;
end;

// Add the sound segment
//
function CGPSoundSegments.AddSoundSegment: CGPSoundSegment;
begin
	// Create new sound segment
	result := CGPSoundSegment.Create(self);

   // Add to segment list
   m_segments.Add(result);
end;

// Copy and add the sound segment
//
function CGPSoundSegments.CopyAndAddSoundSegment(segment: CGPSoundSegment): CGPSoundSegment;
begin
	// Create new sound segment
	result := CGPSoundSegment.Create(self);

   // Copy values from segment
   result.AsString := segment.AsString;

   // Add to segment list
   m_segments.Add(result);
end;

// Delete the segment
//
procedure CGPSoundSegments.Delete(idx: integer);
begin
	m_segments.Delete(idx);
end;

//--------------------------------------------------------------------------
// Private methods
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
// Property methods
//--------------------------------------------------------------------------

// Get the sound segment
//
function CGPSoundSegments.getSoundSegment(idx: integer): CGPSoundSegment;
begin
	result := CGPSoundSegment(m_segments[idx]);
end;

// Get the sound segment count
//
function CGPSoundSegments.getCount: integer;
begin
	result := m_segments.Count;
end;
