//------------------------------------------------------------------------------
// Guitar Practice
//------------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		The guitar practice misc utils.
//
//	Description:   	This unit contains the util functions used by
//                 the guitar practice system
//
// Created:		FeMann
// Date:			30.08.2009
//
// (c) Copyright by FeMann.
// All rights reserved.
//
//

unit CGPUtils;

interface

uses
	SysUtils;

// Convert integer to string
function IntegerToString(val: integer; const fillupLen: integer = - 1; const fillupChar: char = '0'; const fromLeft: boolean = true): string;
// Convert seconds to time sting
function SecondsToTimeString(seconds: integer): string;
// Split seconds to minute and second value
procedure SplitSeconds(seconds: currency; var minutes: integer; var second: integer; var ms: integer);
// Calculate the seconds
function CalculateSeconds(minutes: integer; seconds: integer; ms: integer): currency;
// Calculate bar position from position
function PositionToBarPosition(val: currency): integer;
// Calculate position from bar position
function BarPositionToPosition(val: integer): currency;
// Convert milliseconds to tenth milli seconds
function MilliSecondsToTenthMilliseconds(val: integer): integer;
// Convert milliseconds to spin button value seconds
function MilliSecondsToSpinButtonValue(val: integer): integer;
// Convert tenth milli seconds to milliseconds
function TenthMillisecondsToMilliSeconds(val: integer): integer;
// Include trailing backslah in given path
function PathIncludeTrailingBackslash(path: string): string;
// Convert string to boolean
function StringToBoolean(val: string): boolean;
// Convert boolean to string
function BooleanToString(val: boolean): string;



implementation

// Convert integer to string
//
function IntegerToString(val: integer; const fillupLen: integer = - 1; const fillupChar: char = '0'; const fromLeft: boolean = true): string;
var
	fs: string;
begin
	// Convert value to string
	result := IntToStr(val);

	// Check for fillup
	if (fillupLen <> - 1) then
	begin
		// VAlidate length
		if (Length(result) < fillupLen) then
		begin
			// Calculate fill string
			fs := StringOfChar(fillupChar,fillupLen - Length(result));

			// Build concatenated string
			if fromLeft then
				result := fs + result
			else
				result := result + fs;
		end;
	end;
end;

// Convert seconds to string with format [hh:]mm:ss
//
function SecondsToTimeString(seconds: integer): string;
var
	h,m: integer;
begin
	// Calculate number of hours
	h := trunc(seconds / 3600);
	seconds := seconds - h * 3600;

	// Calculate number of minutes
	m := trunc(seconds / 60);

	// Calculate number of seconds
	seconds := seconds - m * 60;

	// Calculate result
	result := IntegerToString(h,2) + ':' + IntegerToString(m,2) + ':' + IntegerToString(seconds,2);
end;

// Split seconds to minute and second value
//
procedure SplitSeconds(seconds: currency; var minutes: integer; var second: integer; var ms: integer);
begin
	// Calculate number of minutes
	minutes := trunc(seconds / 60);

	// Calculate number of seconds
	second := Trunc(seconds) - (minutes * 60);

	// Calculate number of milli seconds
	ms := Trunc((seconds - ((minutes * 60) + second)) * 10000);
end;

// Calculate the seconds
//
function CalculateSeconds(minutes: integer; seconds: integer; ms: integer): currency;
begin
	result := (minutes * 60) + seconds + (ms / 10000);
end;

// Calculate bar position from position
//
function PositionToBarPosition(val: currency): integer;
begin
	result := Trunc(val * 10000);
end;

// Calculate position from bar position
//
function BarPositionToPosition(val: integer): currency;
begin
	result := val / 10000;
end;

// Convert milliseconds to tenth milli seconds
//
function MilliSecondsToTenthMilliseconds(val: integer): integer;
begin
	result := Trunc(val / 100);
end;

// Convert tenth milli seconds to milliseconds
//
function TenthMillisecondsToMilliSeconds(val: integer): integer;
begin
	result := Trunc(val * 100);
end;

// Convert milli seconds to spin button value
//
function MilliSecondsToSpinButtonValue(val: integer): integer;
begin
	// Round value to 10th milli seconds
	result := (Trunc(val / 100) div 10) * 10;
end;

// Include trailing backslah in given path
//
function PathIncludeTrailingBackslash(path: string): string;
begin
	// Nothing to do
	if Length(path) = 0 then
		exit;

	// Include backslash if necessary
	if path[Length(path)] = '\' then
		result := path
	else
		result := path + '\';
end;

// Convert string to boolean
//
function StringToBoolean(val: string): boolean;
begin
	// Initialize result
	result := false;

	// validate length
	if (Length(val) <> 1) then
		exit;

	// Convert string value to boolean value
	if val = '1' then
		result := true
	else
		result := false;
end;

// Convert boolean to string
//
function BooleanToString(val: boolean): string;
begin
	// Covert boolean value to string value
	if val then
		result := '1'
	else
		result := '0';
end;

end.
