//
// Unit:			CCommandLineManager.pas
// Description:	This unit implements the command line manager class
// Autor:          FeMann
// Date:			11.06.2004
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CCommandLineManager.Create;
begin
	inherited;

   // Create the params
   m_params := TList.Create;
end;

// Destructor
//
destructor CCommandLineManager.Destroy;
begin
	try
   	// Clear the params
       clearParams;

       // Free the containers
   	FreeAndNil(m_params);
   finally
   	inherited;
   end;
end;

//------------------------------------------------------------------------------
// Public declarations
//------------------------------------------------------------------------------

// Parse the command line
//
function CCommandLineManager.Parse: boolean;
var
	i,j: integer;
	name, value: string;
begin
	result := true;

	// Clear the params
   clearParams;

	j := ParamCount;
   i := 1;

	while i <= j do
	begin
		name := UpperCase(ParamStr(i));

       if i = j then
       	raise Exception.Create('Error: Missing parameter value for: ' + name);

       inc(i);

       value := ParamStr(i);

		inc(i);

       if (name <> '') and (value <> '') then
       	addParam(name, value)
       else
       begin
       	result := false;

       	if (name = '') then
           	raise Exception.Create('Error: Parameter name is empty!');

       	if (value = '') then
           	raise Exception.Create('Error: Parameter value is empty!');
       end;
	end;
end;

// Determine if parameter exists
//
function CCommandLineManager.ParamExists(name: string): boolean;
var
	i: integer;
   param: CCommandLineParam;
begin
	result := false;

	for i := 0 to m_params.Count - 1 do
   begin
   	param := CCommandLineParam(m_params[i]);

   	if UpperCase(name) = UpperCase(param.Name) then
       begin
       	result := true;
           exit;
       end;
   end;
end;

//------------------------------------------------------------------------------
// Private declarations
//------------------------------------------------------------------------------

// Clear the params
//
procedure CCommandLineManager.clearParams;
var
	i: integer;
   param: CCommandLineParam;
begin
	for i := 0 to m_params.Count - 1 do
   begin
   	param := CCommandLineParam(m_params[i]);
       if (param <> nil) then
       	FreeAndNil(param);
   end;
   m_params.Clear;
end;

// Add a new parameter
//
procedure CCommandLineManager.addParam(const name: string; const value: string);
var
	param: CCommandLineParam;
begin
   if not ParamExists(name) then
   begin
   	param := CCommandLineParam.Create;

       param.Name := UpperCase(name);
       param.Value := UpperCase(value);

       m_params.Add(param);
   end;
end;

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

function CCommandLineManager.getParamValue(name: string): string;
var
	i: integer;
   param: CCommandLineParam;
begin
	result := '';

	for i := 0 to m_params.Count - 1 do
   begin
   	param := CCommandLineParam(m_params[i]);

   	if UpperCase(name) = UpperCase(param.Name) then
       begin
       	result := param.Value;
           exit;
       end;
   end;
end;

