//------------------------------------------------------------------------------
// File classes
//------------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		The file classes classes.
//
//	Description:   	This unit contains classes for easy file management
//
// Created:		FeMann
// Date:			04.10.2009
//
// (c) Copyright by FeMann.
// All rights reserved.
//
//

unit CFileClasses;

interface

uses
	Classes, ContNrs, Menus;

type
	// Event declarations
   //
   COnPopupItemClickEvent = procedure(Sender: TObject; FileName: string) of object;

	// Forward declarations
	//
   CLastOpenFiles = class;
   CLastOpenFile = class;
   CDelimitedStringList = class;

	//---------------------------------------------------------------------------
	// The open files class
	//---------------------------------------------------------------------------

	 CLastOpenFiles = class
		// Private declarations
		private
       	m_files: TObjectList;
           m_popupMenu: TPopupMenu;
           m_maxFiles: integer;
           m_internalAdd: boolean;
       private
       	m_onPopupItemClickEvent: COnPopupItemClickEvent;
       private
			function getAsString: string;
           procedure setAsString(val: string);
       private
       	procedure onClick(Sender: TObject);
       	procedure removeFile(fileName: string);
           procedure collectGarbage;
           procedure buildPopupMenu;
           function hasFile(fileName: string; var idx: integer): boolean;
           function count: integer;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create;
			destructor Destroy; override;
		public
			procedure Reset;
           procedure Add(fileName: string);
       public
       	property MaxFiles: integer read m_maxFiles write m_maxFiles;
           property PopupMenu: TPopupMenu read m_popupMenu write m_popupMenu;
           property AsString: string read getAsString write setAsString;
           property OnPopupItemClick: COnPopupItemClickEvent read m_onPopupItemClickEvent write m_onPopupItemClickEvent;
	end;

	//---------------------------------------------------------------------------
	// The open file class
	//---------------------------------------------------------------------------

	 CLastOpenFile = class
		// Private declarations
		private
       	m_files: CLastOpenFiles;
           m_fileName: string;
       private
       	function getExists: boolean;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myFiles: CLastOpenFiles);
			destructor Destroy; override;
		public
       	procedure Reset;
       public
       	property FileName: string read m_fileName write m_fileName;
           property Exists: boolean read getExists;
	end;

	//---------------------------------------------------------------------------
	// The delimiter string class
	//---------------------------------------------------------------------------

	CDelimitedStringList = class(TStringList)
   	// Private declarations
       private
       	m_delimiter: char;
       private
       	function getDelimitedText: string;
           procedure setDelimitedText(val: string);
       // Public declarations
 		public
       	constructor Create;
			destructor Destroy; override;
       public
       	property DelimitedText: string read getDelimitedText write setDelimitedText;
           property Delimiter: char read m_delimiter write m_delimiter;
   end;

implementation

uses
	SysUtils;

//------------------------------------------------------------------------------
// Implementation of CLastOpenFiles.pas
//------------------------------------------------------------------------------
{$I CLastOpenFiles.pas}

//------------------------------------------------------------------------------
// Implementation of CLastOpenFile.pas
//------------------------------------------------------------------------------
{$I CLastOpenFile.pas}

//------------------------------------------------------------------------------
// Implementation of CDelimitedStringList.pas
//------------------------------------------------------------------------------
{$I CDelimitedStringList.pas}

end.
