//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CLanguageFile.
//
//	Description:  	This unit implements the language file class
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//


//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CLanguageFile.Create(myManager: CLanguageManager);
begin
	inherited Create;

   // Set manager backward reference
	m_manager := myManager;

	// Reset the file
	Reset;
end;

// Destructor
//
destructor CLanguageFile.Destroy;
begin
	try
   	// Reset manager reference
       m_manager := nil;
   finally
		inherited;
   end;
end;

//------------------------------------------------------------------------------
// Public declarations
//------------------------------------------------------------------------------

// Reset the file
//
procedure CLanguageFile.Reset;
begin
	// Reset the forms
	m_manager.Forms.Reset;
end;

// Load the file
//
procedure CLanguageFile.Load(fileName: string);
begin
	// Reset the language file
	Reset;

	// Check if file exists
	if not FileExists(fileName) then
		exit;

	// Create the file
	m_file := TIniFile.Create(fileName);
	try
		// Load file data
		loadFileData;
	finally
		// Free file
		FreeAndNil(m_file);
	end;
end;

// Save the file
//
procedure CLanguageFile.Save(fileName: string);
begin
	// Create the file
	m_file := TIniFile.Create(fileName);
	try
		// Save file data
		saveFileData;

		// Update the file
		m_file.UpdateFile;
	finally
		// Free file
		FreeAndNil(m_file);
	end;
end;

//------------------------------------------------------------------------------
// Private declarations
//------------------------------------------------------------------------------

// Save the data
//
procedure CLanguageFile.saveFileData;
begin
	// Save the form data
   saveFormData;

   // Save the variable data
   saveVariableData;
end;

// Save the form data
//
procedure CLanguageFile.saveFormData;
var
	i, j: integer;
	translateForm: CTranslateForm;
	translateControl: CTranslateFormControl;
begin
	// Write the forms
	for i := 0 to m_manager.Forms.Count - 1 do
	begin
		// Get the form
		translateForm := m_manager.Forms.Form[i];

		// Write the controls
		for j := 0 to translateForm.Controls.Count - 1 do
		begin
			// Get the control
			translateControl := translateForm.Controls.Control[j];

			// Write the control to file
			m_file.WriteString(translateForm.FormName, translateControl.ControlName, translateControl.Caption);
		end;
	end;
end;

// Save the variable data
//
procedure CLanguageFile.saveVariableData;
var
	i: integer;
	variable: CTranslateVariable;
begin
	// Write the variables
	for i := 0 to m_manager.Variables.Count - 1 do
	begin
		// Get the variable
		variable := m_manager.Variables.Variable[i];

       // Write the variable to file
       m_file.WriteString(CONST_LANGUAGEFILE_SECTION_VARIABLES, variable.Name, variable.Text);
	end;
end;


// Load the data
//
procedure CLanguageFile.loadFileData;
begin
	// Load the form data
   loadFormData;

   // Load the variable data
   loadVariableData;
end;

// Load the form data
//
procedure CLanguageFile.loadFormData;
var
	i,j: integer;
	sections: TStringList;
	sectionValues: TStringList;
	section: string;
	sectionValue: string;
	translateForm: CTranslateForm;
	translateControl: CTranslateFormControl;
	name, value: string;
begin
	// Create the sections
	sections := TStringList.Create;
	// Create the section values
	sectionValues := TStringList.Create;
	try
		// Read the sections
		m_file.ReadSections(sections);

		// Iterate sections
		for i := 0 to sections.Count - 1 do
		begin
			// Get the section name
			section := sections[i];

			// Get the form
			translateForm := m_manager.Forms.FormByName[section];

			// Clear the section values
			sectionValues.Clear;

			// Read the section values
			m_file.ReadSectionValues(section,sectionValues);

			// Iterate section values
			for j := 0 to sectionValues.Count - 1 do
			begin
				// Get the section value
				sectionValue := sectionValues[j];

				// Parse the section value
               parseSectionValue(sectionValue,name,value);

				// Get the control
				translateControl := translateForm.Controls.ControlByName[name];

				// Set the control caption
 				translateControl.Caption := value;
			end;
		end;
	finally
		// Free the sections
		FreeAndNil(sections);

		// Free the section values
		FreeAndNil(sectionValues);
	end;
end;

// Load the variable data
//
procedure CLanguageFile.loadVariableData;
var
	i: integer;
	sectionValues: TStringList;
	sectionValue: string;
	variable: CTranslateVariable;
	name, value: string;
begin
	// Create the section values
	sectionValues := TStringList.Create;
	try
   	// Clear the section values
       sectionValues.Clear;

       // Read the section values
       m_file.ReadSectionValues(CONST_LANGUAGEFILE_SECTION_VARIABLES,sectionValues);

   	// Iterate section values
       for i := 0 to sectionValues.Count - 1 do
       begin
       	// Get the section value
           sectionValue := sectionValues[i];

           // Parse the section value
           parseSectionValue(sectionValue,name,value);

           // Get the variable
           variable := m_manager.Variables.VariableByName[name];

           // Set the variable test
           if (variable <> nil) then
           	variable.Text := value;
       end;
	finally
		// Free the section values
		FreeAndNil(sectionValues);
	end;
end;

// Parse the section value
//
procedure CLanguageFile.parseSectionValue(sectionValue: string; var name: string; var value: string);
var
	posSeparator: integer;
begin
	// Get the separator position
   posSeparator := Pos('=',sectionValue);

   // Get the name
   name := Copy(sectionValue,1,posSeparator - 1);

   // Get the value
   value := Copy(sectionValue,posSeparator + 1,Length(sectionValue) - posSeparator);
end;

//------------------------------------------------------------------------------
// Property declarations
//------------------------------------------------------------------------------

