//------------------------------------------------------------------------------
// CLanguages
//------------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		The language classes.
//
//	Description:   	This unit contains the language classes for multi language
//                 projects
//
// Created:		FeMann
// Date:			02.09.2009
//
// (c) Copyright by FeMann.
// All rights reserved.
//
//

unit CLanguages;

interface

uses
	Windows, Forms, SysUtils, Classes, Controls, Contnrs, IniFiles, Menus;

type
	// Forward declarations
	//
	CLanguageManager = class;
	CLanguageFile = class;
	CTranslateDefinitionBase = class;
	CTranslateTControlCracker = class;
   CTranslateTMenuItemCracker = class;
	CTranslateControls = class;
	CTranslateControl = class;
	CTranslateVariables = class;
	CTranslateVariable = class;
	CTranslateForms = class;
	CTranslateForm = class;
	CTranslateFormControls = class;
	CTranslateFormControl = class;
	CTranslateableForms = class;
	CTranslateableForm = class;
	CTranslations = class;
   CTranslation = class;

	// The translatable form class
	CTranslatableFormClass = class of TForm;

	// The translate definition base class
	CTranslateDefinitionBaseClass = class of CTranslateDefinitionBase;

   // Event declarations
   COnLanguageManagerChangeLanguageEvent = procedure(Sender: TObject; Language: string) of object;

	// The language manager
	//
	CLanguageManager = class
		// Private declarations
		private
			m_controls: CTranslateControls;
			m_variables: CTranslateVariables;
			m_forms: CTranslateForms;
			m_translateableForms: CTranslateableForms;
			m_languageFile: CLanguageFile;
           m_translations: CTranslations;
           m_currentLanguage: string;
			m_translationRegistered: boolean;
           m_translating: boolean;
			m_translationMode: boolean;
       private
       	m_onChangeLanguage: COnLanguageManagerChangeLanguageEvent;
           m_onEndTranslation: TNotifyEvent;
		private
			function getControls: CTranslateControls;
			function getVariables: CTranslateVariables;
			function getForms: CTranslateForms;
			function getTranslateableForms: CTranslateableForms;
			function getLanguageFile: CLanguageFile;
           function getTranslations: CTranslations;
           function getLanguageDirectory: string;
			function getTranslationFileName: string;
		private
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create;
			destructor Destroy; override;
		public
			procedure RegisterTranslation(definitionClass: CTranslateDefinitionBaseClass);
			procedure StartTranslation;
			procedure TranslateForm(frm: TForm);
           procedure ChooseLanguage;
           procedure ChangeLanguage(language: string);
		public
			property Controls: CTranslateControls read getControls;
			property Variables: CTranslateVariables read getVariables;
			property Forms: CTranslateForms read getForms;
			property TranslateableForms: CTranslateableForms read getTranslateableForms;
			property LanguageFile: CLanguageFile read getLanguageFile;
           property LanguageDirectory: string read getLanguageDirectory;
           property Translations: CTranslations read getTranslations;
			property TranslationMode: boolean read m_translationMode write m_translationMode;
           property Translating: boolean read m_translating;
			property TranslationFileName: string read getTranslationFileName;
           property CurrentLanguage: string read m_currentLanguage;
       public
       	property OnChangeLanguage: COnLanguageManagerChangeLanguageEvent read m_onChangeLanguage write m_onChangeLanguage;
           property OnEndTranslation: TNotifyEvent read m_onEndTranslation write m_onEndTranslation;
	end;

	// The language file
	//
	CLanguageFile = class
		// Private declarations
		private
			m_manager: CLanguageManager;
			m_file: TIniFile;
		private
			procedure saveFileData;
           procedure saveFormData;
           procedure saveVariableData;
			procedure loadFileData;
           procedure loadFormData;
           procedure loadVariableData;
           procedure parseSectionValue(sectionValue: string; var name: string; var value: string);
		// Public declarations
		public
			constructor Create(myManager: CLanguageManager);
			destructor Destroy; override;
		public
			procedure Reset;
			procedure Load(fileName: string);
			procedure Save(fileName: string);
	end;

	// The translate definition base class
	//
	CTranslateDefinitionBase = class
		// Private declarations
		private
			m_manager: CLanguageManager;
		// Public declarations
		public
			constructor Create(myManager: CLanguageManager);
			destructor Destroy; override;
		public
			procedure RegisterForms; virtual; abstract;
			procedure RegisterVariables; virtual; abstract;
			procedure RegisterForm(formClass: CTranslatableFormClass; formName: string);
			procedure RegisterVariable(name: string; text: string);
	end;

	// The translate tcontrol cracker
	//
	CTranslateTControlCracker = class(TControl)
		// Public declarations
		public
			property Caption;
	end;

	// The translate tmenuitem cracker
	//
	CTranslateTMenuItemCracker = class(TMenuItem)
		// Public declarations
		public
			property Caption;
	end;

	// The translateable controls
	//
	CTranslateControls = class
		// Private declarations
		private
			m_manager: CLanguageManager;
			m_controls: TObjectList;
		private
			function getControl(idx: integer): CTranslateControl;
           function getControlByName(name: string): CTranslateControl;
			function getCount: integer;
       private
       	function isTranslateableControl(component: TComponent): boolean;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myManager: CLanguageManager);
			destructor Destroy; override;
		public
			procedure Reset;
			procedure UndoChanges;
			function LoadFromForm(frm: TForm): boolean;
		public
			property Control[idx: integer]: CTranslateControl read getControl; default;
           property ControlByName[name: string]: CTranslateControl read getControlByName;
			property Count: integer read getCount;
	end;

	// The translateable control
	//
	CTranslateControl = class
		// Private declarations
		private
			m_controls: CTranslateControls;
			m_component: TComponent;
			m_originalCaption: string;
		private
			function getName: string;
			function getValid: boolean;
			function getComponent: TComponent;
			procedure setComponent(val: TComponent);
			function getCaption: string;
			procedure setCaption(val: string);
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myControls: CTranslateControls);
			destructor Destroy; override;
		public
			procedure Reset;
			procedure UndoChanges;
		public
			property Component: TComponent read getComponent write setComponent;
			property Name: string read getName;
			property Caption: string read getCaption write setCaption;
			property OriginalCaption: string read m_originalCaption;
			property Valid: boolean read getValid;
	end;

	// The translateable variables
	//
	CTranslateVariables = class
		// Private declarations
		private
			m_manager: CLanguageManager;
			m_variables: TObjectList;
		private
			function getVariable(idx: integer): CTranslateVariable;
           function getVariableByName(name: string): CTranslateVariable;
			function getText(name: string): string;
			function getHasVariable(name: string): boolean;
			function getCount: integer;
		private
			function lookupVariableByName(name: string; var v: CTranslateVariable): boolean;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myManager: CLanguageManager);
			destructor Destroy; override;
		public
			procedure Clear;
			procedure UndoChanges;
			procedure AddVariable(name: string; text: string);
		public
			property Variable[idx: integer]: CTranslateVariable read getVariable;
           property VariableByName[name: string]: CTranslateVariable read getVariableByName;
			property Text[name: string]: string read getText; default;
			property HasVariable[name: string]: boolean read getHasVariable;
			property Count: integer read getCount;
	end;

	// The translateable variable
	//
	CTranslateVariable = class
		// Private declarations
		private
			m_variables: CTranslateVariables;
			m_name: string;
			m_text: string;
			m_originalText: string;
		private
			function getText: string;
			procedure setText(val: string);
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myVariables: CTranslateVariables);
			destructor Destroy; override;
		public
			procedure Reset;
			procedure UndoChanges;
		public
			property Name: string read m_name write m_name;
			property Text: string read getText write setText;
			property OriginalText: string read m_originalText write m_originalText;
	end;

	// The translateable forms container
	//
	CTranslateForms = class
		// Private declarations
		private
			m_manager: CLanguageManager;
			m_forms: TObjectList;
		private
			function getForm(idx: integer): CTranslateForm;
			function getFormByName(name: string): CTranslateForm;
			function getHasForm(name: string): boolean;
			function getCount: integer;
		private
			function lookupFormByName(name: string; var frm: CTranslateForm): boolean;
			function translateFormName(name: string): string;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myManager: CLanguageManager);
			destructor Destroy; override;
		public
			procedure Reset;
		public
			property Form[idx: integer]: CTranslateForm read getForm;
			property FormByName[name: string]: CTranslateForm read getFormByName; default;
			property HasForm[name: string]: boolean read getHasForm;
			property Count: integer read getCount;
	end;

	// The translatable form
	//
	CTranslateForm = class
		// Private declarations
		private
			m_forms: CTranslateForms;
			m_formName: string;
			m_controls: CTranslateFormControls;
		private
			procedure freeControls;
		private
			function getControls: CTranslateFormControls;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myForms: CTranslateForms);
			destructor Destroy; override;
		public
			procedure Reset;
		public
			property FormName: string read m_formName write m_formName;
			property Controls: CTranslateFormControls read getControls;
	end;

	// The translateable forms controls
	//
	CTranslateFormControls = class
		// Private declarations
		private
			m_form: CTranslateForm;
			m_controls: TObjectList;
		private
			function getControl(idx: integer): CTranslateFormControl;
			function getControlByName(name: string): CTranslateFormControl;
			function getHasControl(name: string): boolean;
			function getCount: integer;
		private
			function lookupControlByName(name: string; var ctrl: CTranslateFormControl): boolean;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myForm: CTranslateForm);
			destructor Destroy; override;
		public
			procedure Reset;
		public
			property Control[idx: integer]: CTranslateFormControl read getControl;
			property ControlByName[name: string]: CTranslateFormControl read getControlByName; default;
			property HasControl[name: string]: boolean read getHasControl;
			property Count: integer read getCount;
	end;

	// The translatable form control
	//
	CTranslateFormControl = class
		// Private declarations
		private
			m_formControls: CTranslateFormControls;
			m_controlName: string;
			m_caption: string;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myControls: CTranslateFormControls);
			destructor Destroy; override;
		public
			procedure Reset;
		public
			property ControlName: string read m_controlName write m_controlName;
			property Caption: string read m_caption write m_caption;
	end;

	// The translateable forms container
	//
	CTranslateableForms = class
		// Private declarations
		private
			m_manager: CLanguageManager;
			m_forms: TObjectList;
		private
			function getForm(idx: integer): CTranslateableForm;
			function getCount: integer;
		private
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myManager: CLanguageManager);
			destructor Destroy; override;
		public
			procedure Reset;
			procedure HideAll;
			procedure RegisterForm(formClass: CTranslatableFormClass; formName: string);
		public
			property Form[idx: integer]: CTranslateableForm read getForm;
			property Count: integer read getCount;
	end;

	// The translatable form
	//
	CTranslateableForm = class
		// Private declarations
		private
			m_forms: CTranslateableForms;
			m_formClass: CTranslatableFormClass;
			m_formInstance: TForm;
			m_formName: string;
		private
			function getFormInstance: TForm;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myForms: CTranslateableForms);
			destructor Destroy; override;
		public
			procedure Reset;
			procedure Hide;
			procedure ReleaseFormInstance;
		public
			property FormClass: CTranslatableFormClass read m_formClass write m_formClass;
			property FormInstance: TForm read getFormInstance;
			property FormName: string read m_formName write m_formName;
	end;

	// The available translations
	//
	CTranslations = class
		// Private declarations
		private
			m_manager: CLanguageManager;
			m_translations: TObjectList;
		private
			function getTranslation(idx: integer): CTranslation;
           function getTranslationByName(name: string): CTranslation;
			function getCount: integer;
       private
       	procedure scanDirectory(const path: string; filemask: string; hitlist: TStrings);
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myManager: CLanguageManager);
			destructor Destroy; override;
		public
			procedure Reset;
           function Load: boolean;
		public
			property Translation[idx: integer]: CTranslation read getTranslation; default;
           property TranslationByName[name: string]: CTranslation read getTranslationByName;
			property Count: integer read getCount;
	end;

	// The available translation
	//
	CTranslation = class
		// Private declarations
		private
			m_translations: CTranslations;
           m_fileName: string;
		private
			function getName: string;
		// Protected declarations
		protected
		// Public declarations
		public
			constructor Create(myTranslations: CTranslations);
			destructor Destroy; override;
		public
			procedure Reset;
		public
			property Name: string read getName;
			property FileName: string read m_fileName write m_fileName;
	end;

	// Get the language manager instance
	function theLanguageManager: CLanguageManager;
	// Release the language manager instance
	procedure ReleaseLanguageManager;
	// Include trailing backslah in given path
	function PathIncludeTrailingBackslash(path: string): string;

const
	CONST_LANGUAGEFOLDER_LANGUAGE: string = 'language';
	CONST_LANGUAGEFOLDER_TRANSLATION: string = 'translation';
	CONST_LANGUAGEFILE_TRANSLATION: string = 'translation.lf';
   CONST_LANGUAGEFILE_SECTION_VARIABLES: string = 'Variables';
   CONST_LANGUAGEFILE_EXTENSION: string = '.lf';
   CONST_DEFAULT_LANGUAGE: string = '<Default language>';

implementation

uses
	UFrmTranslateForm, UFrmChooseTranslation, StdCtrls, ExtCtrls, Spin, ComCtrls,
   // FileCtrl is platform dependant
   {$WARN UNIT_PLATFORM OFF}FileCtrl{$WARN UNIT_PLATFORM ON};

var
	myLanguageManager: CLanguageManager;

	// Get the language manager instance
	//
	function theLanguageManager: CLanguageManager;
	begin
		// Create language manager on demand
		if not Assigned(myLanguageManager) then
			myLanguageManager := CLanguageManager.Create;

		// Return language manager instance
		result := myLanguageManager;
	end;

	// Release the language manager
	//
	procedure ReleaseLanguageManager;
	begin
		// Free language manager instance
		if Assigned(myLanguageManager) then
			FreeAndNil(myLanguageManager);
	end;

	// Include trailing backslah in given path
	//
	function PathIncludeTrailingBackslash(path: string): string;
	begin
		// Nothing to do
		if Length(path) = 0 then
			exit;

		// Include backslash if necessary
		if path[Length(path)] = '\' then
			result := path
		else
			result := path + '\';
	end;

//------------------------------------------------------------------------------
// Implementation of CLanguageManager.pas
//------------------------------------------------------------------------------
{$I CLanguageManager.pas}

//------------------------------------------------------------------------------
// Implementation of CLanguageFile.pas
//------------------------------------------------------------------------------
{$I CLanguageFile.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateDefinitionBase.pas
//------------------------------------------------------------------------------
{$I CTranslateDefinitionBase.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateControls.pas
//------------------------------------------------------------------------------
{$I CTranslateControls.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateControl.pas
//------------------------------------------------------------------------------
{$I CTranslateControl.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateVariables.pas
//------------------------------------------------------------------------------
{$I CTranslateVariables.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateVariable.pas
//------------------------------------------------------------------------------
{$I CTranslateVariable.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateForms.pas
//------------------------------------------------------------------------------
{$I CTranslateForms.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateForm.pas
//------------------------------------------------------------------------------
{$I CTranslateForm.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateFormControls.pas
//------------------------------------------------------------------------------
{$I CTranslateFormControls.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateFormControl.pas
//------------------------------------------------------------------------------
{$I CTranslateFormControl.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateableForms.pas
//------------------------------------------------------------------------------
{$I CTranslateableForms.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslateableForm.pas
//------------------------------------------------------------------------------
{$I CTranslateableForm.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslations.pas
//------------------------------------------------------------------------------
{$I CTranslations.pas}

//------------------------------------------------------------------------------
// Implementation of CTranslation.pas
//------------------------------------------------------------------------------
{$I CTranslation.pas}

// Initialization
initialization

	// Init the language manager variable
	myLanguageManager := nil;

// Finalization
finalization

	// Release the language manager
	ReleaseLanguageManager;

end.
