//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CTranslateControl.
//
//	Description:  	This unit implements the translatable control  class
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CTranslateControl.Create(myControls: CTranslateControls);
begin
	inherited Create;

	// Set backward reference to controls
	m_controls := myControls;
end;

// Destructor
//
destructor CTranslateControl.Destroy;
begin
	try
		// Reset the controls reference
		m_controls := nil;
	finally
   	inherited;
   end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Reset the control
//
procedure CTranslateControl.Reset;
begin
	m_component := nil;
	m_originalCaption := '';
end;

// Undo the control changes
//
procedure CTranslateControl.UndoChanges;
begin
	// Restore the original properties
   Caption := OriginalCaption;
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the component
//
function CTranslateControl.getComponent: TComponent;
begin
	result := m_component;
end;

// Set the component
//
procedure CTranslateControl.setComponent(val: TComponent);
begin
	// Set control
	m_component := val;

	// Validate control
	if Valid then
	begin
		// Store original properties
		m_originalCaption := Caption;
	end;
end;

// Get the control name
//
function CTranslateControl.getName: string;
begin
	// Initialize result
	result := '';

	// Get the name from the control
	if Valid then
		result := m_component.Name;
end;

// Get the control caption
//
function CTranslateControl.getCaption: string;
begin
	// Initialize result
	result := '';

	// Get the caption from the control
	if Valid then
   begin
   	if (m_component is TControl) then
			result := CTranslateTControlCracker(m_component).Caption
       else if (m_component is TMenuItem) then
       	result := CTranslateTMenuItemCracker(m_component).Caption
       else
       	raise Exception.Create('[CTranslateControl.getCaption]: Unsupported component type: ' + m_component.Name);
   end;
end;

// Set the control caption
//
procedure CTranslateControl.setCaption(val: string);
begin
	// Set the caption from the control
	if Valid then
	begin
      	if (m_component is TControl) then
			CTranslateTControlCracker(m_component).Caption := val
       else if (m_component is TMenuItem) then
       	CTranslateTMenuItemCracker(m_component).Caption := val
       else
       	raise Exception.Create('[CTranslateControl.setCaption]: Unsupported component type: ' + m_component.Name);
   end;
end;

// Determine if control is valid
//
function CTranslateControl.getValid: boolean;
begin
	result := (m_component <> nil) and (m_component.Name <> '');
end;
