//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CTranslateControls.
//
//	Description:  	This unit implements the translatable contols class
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CTranslateControls.Create(myManager: CLanguageManager);
begin
	inherited Create;

	// Set backwards reference to manager
	m_manager := myManager;

	// Create the control container
	m_controls := TObjectList.Create;
end;

// Destructor
//
destructor CTranslateControls.Destroy;
begin
	try
		// Free the control container
		FreeAndNil(m_controls);

		// Reset manager reference
		m_manager := nil;
	finally
		inherited;
	end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Reset the controls
//
procedure CTranslateControls.Reset;
begin
	// Clear controls
	m_controls.Clear;
end;

// Undo changes
//
procedure CTranslateControls.UndoChanges;
var
	i: integer;
	control: CTranslateControl;
begin
	// Iterate controls
	for i := 0 to Count - 1 do
	begin
		// Get the control
		control := getControl(i);

		// Undo the changes
		control.UndoChanges;
	end;
end;

// Load the controls from form
//
function CTranslateControls.LoadFromForm(frm: TForm): boolean;
var
	component: TComponent;
	control: CTranslateControl;
	i: integer;
begin
	// Initialize result
	result := false;

	// Validate
	if (frm = nil) then
		exit;

	// Reset controls
	Reset;

	// Iterate all form components
	for i := 0 to frm.ComponentCount - 1 do
	begin
		// Get the component
		component := frm.Components[i];

		// Take all controls
		if isTranslateableControl(component) then
		begin
			// Create new control
			control := CTranslateControl.Create(self);

			// Setup the control
			control.Component := component;

			// Check if control is valid
			if control.Valid then
				m_controls.Add(control)
			else
				FreeAndNil(control);
		end;
	end;

	// Check if there are any controls to translate
	result := (m_controls.Count > 0);
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

// Check if component is an translateable control
//
function CTranslateControls.isTranslateableControl(component: TComponent): boolean;
begin
	// Allow all controls
	result := (component is TControl) or (component is TMenuItem);

   // Revoke special controls
   result := result and not(component is TEdit) and not(component is TSpinEdit)
   				 and not(component is TPanel) and not(component is TScrollBar)
                    and not(component is TListView) and not(component is TTreeView);
end;

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the control
//
function CTranslateControls.getControl(idx: integer): CTranslateControl;
begin
	result := CTranslateControl(m_controls[idx]);
end;

// Get the control by name
//
function CTranslateControls.getControlByName(name: string): CTranslateControl;
var
   i: integer;
	uc: string;
   control: CTranslateControl;
begin
	// Initialize result
   result := nil;

   // Get uppercase name
   uc := UpperCase(name);

   // Iterate controls
   for i := 0 to Count - 1 do
   begin
   	// Get the control
       control := getControl(i);

       // Compare name
       if (UpperCase(control.Name) = uc) then
       begin
       	// Control found
       	result := control;

           // Nothing more to do
           exit;
       end;
   end;
end;

// Get the control count
//
function CTranslateControls.getCount: integer;
begin
	result := m_controls.Count;
end;

