//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CTranslateFormControls.
//
//	Description:  	This unit implements the translatable form controls class
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CTranslateFormControls.Create(myForm: CTranslateForm);
begin
	inherited Create;

	// Set backwards reference to manager
	m_form := myForm;

	// Create the controls container
	m_controls := TObjectList.Create;
end;

// Destructor
//
destructor CTranslateFormControls.Destroy;
begin
	try
		// Free the controls container
		FreeAndNil(m_controls);

		// Reset form reference
		m_form := nil;
	finally
		inherited;
	end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Reset the controls
//
procedure CTranslateFormControls.Reset;
begin
	// Clear controls
	m_controls.Clear;
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

// Lookup the given control by name
//
function CTranslateFormControls.lookupControlByName(name: string; var ctrl: CTranslateFormControl): boolean;
var
	lookupControl: CTranslateFormControl;
	i: integer;
begin
	// Initialize result
	ctrl := nil;

	for i := 0 to Count - 1 do
	begin
		// Get the control
		lookupControl := getControl(i);

		// Compare name
		if (UpperCase(lookupControl.ControlName) = UpperCase(name)) then
		begin
			// Return the control
			ctrl := lookupControl;

			// Nothing more to do
			break;
		end;
	end;

	// Determine sucess
	result := (ctrl <> nil);
end;

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the form
//
function CTranslateFormControls.getControl(idx: integer): CTranslateFormControl;
begin
	result := CTranslateFormControl(m_controls[idx]);
end;

// Get the text
//
function CTranslateFormControls.getControlByName(name: string): CTranslateFormControl;
var
	ctrl: CTranslateFormControl;
begin
	// Lookup control
	if not lookupControlByName(name,ctrl) then
	begin
		// Create new control
		ctrl := CTranslateFormControl.Create(self);

		// Setup control name
		ctrl.ControlName := name;

		// Add to controls container
		m_controls.Add(ctrl);
	end;

	// Return control
	result := ctrl;
end;

// Determine if control already exists
//
function CTranslateFormControls.getHasControl(name: string): boolean;
var
	ctrl: CTranslateFormControl;
begin
	result := (lookupControlByName(name,ctrl));
end;

// Get the controls count
//
function CTranslateFormControls.getCount: integer;
begin
	result := m_controls.Count;
end;

