//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CTranslateForms.
//
//	Description:  	This unit implements the translatable forms class
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CTranslateForms.Create(myManager: CLanguageManager);
begin
	inherited Create;

	// Set backwards reference to manager
	m_manager := myManager;

	// Create the forms container
	m_forms := TObjectList.Create;
end;

// Destructor
//
destructor CTranslateForms.Destroy;
begin
	try
		// Free the forms container
		FreeAndNil(m_forms);

		// Reset manager reference
		m_manager := nil;
	finally
		inherited;
	end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Reset the forms
//
procedure CTranslateForms.Reset;
begin
	// Clear forms
	m_forms.Clear;
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

// Translate the form name
//
function CTranslateForms.translateFormName(name: string): string;
begin
	// Strip the delphi generated name
	if (Pos('_',name) <> 0) then
		result := Copy(name,1,Pos('_',name) - 1)
	else
		result := name;
end;

// Lookup the given form by name
//
function CTranslateForms.lookupFormByName(name: string; var frm: CTranslateForm): boolean;
var
	lookupForm: CTranslateForm;
	translatedFormName: string;
	i: integer;
begin
	// Initialize result
	frm := nil;

	// Translate the form name
	translatedFormName := translateFormName(name);

	for i := 0 to Count - 1 do
	begin
		// Get the form
		lookupForm := getForm(i);

		// Compare name
		if (UpperCase(lookupForm.FormName) = UpperCase(translatedFormName)) then
		begin
			// Return the form
			frm := lookupForm;

			// Nothing more to do
			break;
		end;
	end;

	// Determine sucess
	result := (frm <> nil);
end;

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the form
//
function CTranslateForms.getForm(idx: integer): CTranslateForm;
begin
	result := CTranslateForm(m_forms[idx]);
end;

// Get the text
//
function CTranslateForms.getFormByName(name: string): CTranslateForm;
var
	frm: CTranslateForm;
begin
	// Lookup form
	if not lookupFormByName(name,frm) then
	begin
		// Create new form
		frm := CTranslateForm.Create(self);

		// Setup form name
		frm.FormName := translateFormName(name);

		// Add to form container
		m_forms.Add(frm);
	end;

	// Return variable text
	result := frm;
end;

// Determine if form already exists
//
function CTranslateForms.getHasForm(name: string): boolean;
var
	frm: CTranslateForm;
begin
	result := (lookupFormByName(name,frm));
end;

// Get the forms count
//
function CTranslateForms.getCount: integer;
begin
	result := m_forms.Count;
end;

