//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CTranslateVariables.
//
//	Description:  	This unit implements the translatable variables class
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CTranslateVariables.Create(myManager: CLanguageManager);
begin
	inherited Create;

	// Set backwards reference to manager
	m_manager := myManager;

	// Create the variables container
	m_variables := TObjectList.Create;
end;

// Destructor
//
destructor CTranslateVariables.Destroy;
begin
	try
		// Free the variables container
		FreeAndNil(m_variables);

		// Reset manager reference
		m_manager := nil;
	finally
		inherited;
	end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Reset the controls
//
procedure CTranslateVariables.Clear;
begin
	// Clear variables
	m_variables.Clear;
end;

// Undo changes
//
procedure CTranslateVariables.UndoChanges;
var
	i: integer;
	v: CTranslateVariable;
begin
	// Iterate variables
	for i := 0 to Count - 1 do
	begin
		// Get the variable
		v := getVariable(i);

		// Undo the changes
		v.UndoChanges;
	end;
end;

// Add the given variable
//
procedure CTranslateVariables.AddVariable(name: string; text: string);
var
	v: CTranslateVariable;
begin
	// Check if variable already exists
	if HasVariable[name] then
		exit;

	// Create new variable
	v := CTranslateVariable.Create(self);

	// Setup the variable properties
	v.Name := name;
	v.Text := text;

	// Add to variable container
	m_variables.Add(v);
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

// Lookup the given variable by name
//
function CTranslateVariables.lookupVariableByName(name: string; var v: CTranslateVariable): boolean;
var
	lookupVariable: CTranslateVariable;
	i: integer;
begin
	// Initialize result
	v := nil;

	for i := 0 to Count - 1 do
	begin
		// Get the variable
		lookupVariable := getVariable(i);

		// Compare name
		if (UpperCase(lookupVariable.Name) = UpperCase(name)) then
		begin
			// Return the text
			v := lookupVariable;

			// Nothing more to do
			break;
		end;
	end;

	// Determine sucess
	result := (v <> nil);
end;

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the variable
//
function CTranslateVariables.getVariable(idx: integer): CTranslateVariable;
begin
	result := CTranslateVariable(m_variables[idx]);
end;

// Get the variable by name
//
function CTranslateVariables.getVariableByName(name: string): CTranslateVariable;
begin
	lookupVariableByName(name,result);
end;

// Get the text
//
function CTranslateVariables.getText(name: string): string;
var
	v: CTranslateVariable;
begin
	// Lookup variable
	if not lookupVariableByName(name,v) then
		// Raise not found exception
		raise Exception.Create('[CTranslateVariables.getText]: Variable [' + name + '] not found!');

	// Return variable text
	result := v.Text;
end;

// Determine if variable already exists
//
function CTranslateVariables.getHasVariable(name: string): boolean;
var
	v: CTranslateVariable;
begin
   result := (lookupVariableByName(name,v));
end;

// Get the variable count
//
function CTranslateVariables.getCount: integer;
begin
	result := m_variables.Count;
end;

