//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CTranslateableForm.
//
//	Description:  	This unit implements the translatable form class
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CTranslateableForm.Create(myForms: CTranslateableForms);
begin
	inherited Create;

	// Set backward reference to forms
	m_forms := myForms;

	// Reset the form
	Reset;
end;

// Destructor
//
destructor CTranslateableForm.Destroy;
begin
	try
		// Release the form instance
		ReleaseFormInstance;

		// Reset the forms reference
		m_forms := nil;
	finally
		inherited;
	end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Reset the form
//
procedure CTranslateableForm.Reset;
begin
	m_formClass := nil;
	m_formInstance := nil;
	m_formName := '';
end;

// Release the form instance
//
procedure CTranslateableForm.ReleaseFormInstance;
begin
	// Free the form instance
	if Assigned(m_formInstance) then
		FreeAndNil(m_formInstance);
end;

// Hide the form
//
procedure CTranslateableForm.Hide;
begin
	// Hide the form instance
	if Assigned(m_formInstance) and m_formInstance.Showing then
		m_formInstance.Hide;
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the form instance
//
function CTranslateableForm.getFormInstance: TForm;
begin
	// Validate
	if (m_formClass = nil) then
		raise Exception.Create('[CTranslateableForm.getFormInstance]: Form class not set!');

	// Create the form instance on demand
	if not Assigned(m_formInstance) then
		m_formInstance := m_formClass.Create(nil);

	// Return instance
	result := m_formInstance;
end;

