//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CTranslateableForms.
//
//	Description:  	This unit implements the translatable forms class
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CTranslateableForms.Create(myManager: CLanguageManager);
begin
	inherited Create;

	// Set backwards reference to manager
	m_manager := myManager;

	// Create the forms container
	m_forms := TObjectList.Create;
end;

// Destructor
//
destructor CTranslateableForms.Destroy;
begin
	try
		// Free the forms container
		FreeAndNil(m_forms);

		// Reset manager reference
		m_manager := nil;
	finally
		inherited;
	end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Reset the forms
//
procedure CTranslateableForms.Reset;
begin
	// Clear forms
	m_forms.Clear;
end;

// Hide all forms
//
procedure CTranslateableForms.HideAll;
var
	frm: CTranslateableForm;
	i: integer;
begin
	// Iterate all forms
	for i := 0 to Count - 1 do
	begin
		// Get the form
		frm := Form[i];

		// Hide the form
       frm.Hide;
	end;
end;

// Register the form
//
procedure CTranslateableForms.RegisterForm(formClass: CTranslatableFormClass; formName: string);
var
	frm: CTranslateableForm;
begin
	// Create new form
	frm := CTranslateableForm.Create(self);

	// Set the properties
	frm.FormClass := formClass;
	frm.FormName := formName;

	// Add to container
	m_forms.Add(frm);
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the form
//
function CTranslateableForms.getForm(idx: integer): CTranslateableForm;
begin
	result := CTranslateableForm(m_forms[idx]);
end;

// Get the forms count
//
function CTranslateableForms.getCount: integer;
begin
	result := m_forms.Count;
end;

