//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of CTranslations.
//
//	Description:  	This unit implements the translations class
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

//------------------------------------------------------------------------------
// Constructor and Destructor
//------------------------------------------------------------------------------

// Constructor
//
constructor CTranslations.Create(myManager: CLanguageManager);
begin
	inherited Create;

	// Set backwards reference to manager
	m_manager := myManager;

	// Create the translations container
	m_translations := TObjectList.Create;
end;

// Destructor
//
destructor CTranslations.Destroy;
begin
	try
		// Free the translations
		FreeAndNil(m_translations);

		// Reset manager reference
		m_manager := nil;
	finally
		inherited;
	end;
end;

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Reset the controls
//
procedure CTranslations.Reset;
begin
	// Clear controls
	m_translations.Clear;
end;

// Load the translations
//
function CTranslations.Load: boolean;
var
	files: TStringList;
   translation: CTranslation;
   i: integer;
begin
	// Reset the translations
   Reset;

   // Create file list
   files := TStringList.Create;
   try
   	// Scan for language files
   	scanDirectory(m_manager.LanguageDirectory + '\','*' + CONST_LANGUAGEFILE_EXTENSION,files);

       // Iterate files
       for i := 0 to files.Count - 1 do
       begin
       	// Create new translation
   		translation := CTranslation.Create(self);

           // Set the filename
           translation.FileName := files[i];

           // Add to container
           m_translations.Add(translation);
       end;
   finally
   	// Free file list
       FreeAndNil(files);
   end;

   // Determine success
   result := (Count > 0);
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

// Scan the given directory
//
procedure CTranslations.scanDirectory(const path: string; filemask: string; hitlist: TStrings);
var
	SRec: TSearchRec;
   res: Integer;
begin
	// Clear hit list
	hitlist.Clear;

   // Find first file
   res := FindFirst(path + filemask, faAnyfile, SRec);
   if res = 0 then
   try
   	while res = 0 do
       begin
       	hitlist.Add(path + SRec.Name);
          	res := FindNext(SRec);
       end;
   finally
   	FindClose(SRec)
   end;
end;

//------------------------------------------------------------------------------
// Property methods
//------------------------------------------------------------------------------

// Get the translation
//
function CTranslations.getTranslation(idx: integer): CTranslation;
begin
	result := CTranslation(m_translations[idx]);
end;

// Get the translation by name
//
function CTranslations.getTranslationByName(name: string): CTranslation;
var
   i: integer;
	uc: string;
   translation: CTranslation;
begin
	// Initialize result
   result := nil;

   // Get uppercase name
   uc := UpperCase(name);

   // Iterate controls
   for i := 0 to Count - 1 do
   begin
   	// Get the translation
       translation := getTranslation(i);

       // Compare name
       if (UpperCase(translation.Name) = uc) then
       begin
       	// Translation found found
       	result := translation;

           // Nothing more to do
           exit;
       end;
   end;
end;

// Get the control count
//
function CTranslations.getCount: integer;
begin
	result := m_translations.Count;
end;

