//--------------------------------------------------------------------------
// CLanguages
//--------------------------------------------------------------------------
//
//	Version:		1.0
//
//	Synopsis:		Implementation of TFrmChooseTranslation.
//
//	Description:  	This unit implements the user interface to choose the
//					language / translation
//
//
// Author(s):		FeMann
// Created:		02.09.2009
//
// (c) Copyright by FeMann
// All rights reserved.
//
//

unit UFrmChooseTranslation;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
   CLanguages, ComCtrls, StdCtrls, ExtCtrls;

type
	// The language / translation change form
	//
	TFrmChooseTranslation = class(TForm)
    	pnlChooseTranslationBase: TPanel;
    	grpTranslations: TGroupBox;
    	lvwTranslations: TListView;
    	cmdOk: TButton;
    	cmdCancel: TButton;
       // Private declarations
   	private
       	m_manager: CLanguageManager;
       private
       	procedure loadTranslations;
           function getLanguageIndex(name: string): integer;
       // Public declarations
  		public
       	function Execute(manager: CLanguageManager): boolean;
  	end;

var
  FrmChooseTranslation: TFrmChooseTranslation;

implementation

{$R *.DFM}

//------------------------------------------------------------------------------
// Public methods
//------------------------------------------------------------------------------

// Change the language
//
function TFrmChooseTranslation.Execute(manager: CLanguageManager): boolean;
var
	currentLanguageIndex: integer;
begin
	// Set manager reference
	m_manager := manager;

	// Load the translations
   loadTranslations;

   // Try to locate the current language
   currentLanguageIndex := getLanguageIndex(m_manager.CurrentLanguage);

   // Select current language
   if (currentLanguageIndex <> -1) then
   	lvwTranslations.Items[currentLanguageIndex].Selected := true;

	// Determine success
	result := (ShowModal = mrOk) and (lvwTranslations.Selected <> nil);

	// Determine success
	if result then
	begin
   	// Change the language
       m_manager.ChangeLanguage(lvwTranslations.Selected.Caption);
	end;
end;

//------------------------------------------------------------------------------
// Private methods
//------------------------------------------------------------------------------

// Load the translations
//
procedure TFrmChooseTranslation.loadTranslations;
var
	translation: CTranslation;
	item: TListItem;
	i: integer;
begin
	// Reset list view
   lvwTranslations.Items.Clear;

   // Add default language entry
   item := lvwTranslations.Items.Add;

   // Setup item properties
   item.Caption := CONST_DEFAULT_LANGUAGE;
   item.Data := nil;

   // Iterate translations
   for i := 0 to m_manager.Translations.Count - 1 do
   begin
   	// Get the translation
       translation := m_manager.Translations[i];

       // Add default language entry
       item := lvwTranslations.Items.Add;

       // Setup item properties
       item.Caption := translation.Name;
       item.Data := translation;
   end;
end;

// Get the language index
//
function TFrmChooseTranslation.getLanguageIndex(name: string): integer;
var
	item: TListItem;
	i: integer;
begin
	// Initialize result
	result := -1;

   for i := 0 to lvwTranslations.Items.Count - 1 do
   begin
   	// Get the item
   	item := lvwTranslations.Items[i];

       // Compare caption
       if (item.Caption = name) then
       begin
       	// Get the item index
       	result := item.Index;

           // Nothing more to do
           exit;
       end;
   end;
end;

end.
